/* \file vec2.cpp
 *
 */
#include "stdafx.h"
#include <stdio.h>
#include <math.h>
#include <iostream>
#include "vec2.h"

//! overloaded insertion operator <<
std::ostream& operator << (std::ostream& s, const vec2& v)
{
	s << "(" << v[0] << ", "  << v[1] << ")";
	return s ;
}


//! return magnitude of cross product
/*!
 *  for 2D this is dot(perp(a),b)\n
 *  for 3D this is the magnitude of cross(a,b)
 */
double perpdot(const vec2 &a, const vec2 &b)
{
	return a[0]*b[1]-a[1]*b[0];
}

//! returns angle (in degrees) between two vectors
double angle(const vec2 &a, const vec2 &b)
{
	double c, s;
	const double dgr = 45.0/atan(1.0);
	c = dot(a,b);
	s = perpdot(a,b);
	return dgr*atan2(s,c);
}


//! returns vec rotated by angle theta in degrees
vec2 rotate(const vec2 &a, double theta)
{
	double c, s;
	const double radg = atan(1.0)/45.0;
	theta *= radg;
	c = cos(theta);
	s = sin(theta);
	return vec2(c*a[0]-s*a[1],s*a[0]+c*a[1]);
}
